/* 
 * fbxid.c - Prints the framebuffer id string then exits.
 *
 * By Michael Bourgeous
 *
 * $Id: fbxid.c,v 1.8 2001/01/17 22:08:24 lethal Exp $
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <libfbx/libfbx.h>

static char *tmp = NULL;

static void get_id()
{
	/*
	 * This might break on some compilers if for some
	 * reason fb_screen->driver->id is NULL. GCC will
	 * still handle a NULL pointer as "(null)".
	 */
	tmp = (char *)malloc(strlen(fb_screen->driver->id) + 7);
	snprintf(tmp, strlen(fb_screen->driver->id) + 7, "ID: \"%s\"",
                 fb_screen->driver->id);
}

static void get_dev()
{
	/*
	 * Same deal as above, except here we're concerened
	 * about fb_finddev().
	 */
	tmp = (char *)malloc(strlen(fb_finddev()) + 12);
	snprintf(tmp, strlen(fb_finddev()) + 12, "Device: \"%s\"",
		 fb_finddev());
}

static void usage(char *progname)
{
	fprintf(stderr, "Usage: %s [-id]\n", progname);
}

int main(int argc, char *argv[])
{
	if (argc != 2) {
		usage(argv[0]);	
		return 1;
	}

	fb_init();

	if (strstr(argv[1], "-i") != NULL) {
		get_id();
	} else if (strstr(argv[1], "-d") != NULL) {
		get_dev();
	} else {
		fb_cleanup();
		usage(argv[0]);
		return 1;
	}

	fb_cleanup();

	fprintf(stdout, "%s\n", tmp);

	if (tmp != NULL)
		free(tmp);

	return 0;
}

